package no.integration.svv.utils.mapper;

import no.geointegrasjon.rep.arkiv.dokument.xml_schema._2012_01.Dokument;
import no.geointegrasjon.rep.arkiv.oppdatering.xml_wsdl._2012_01.NyDokument;
import no.geointegrasjon.rep.arkiv.oppdatering.xml_wsdl._2012_01.NyDokumentResponse;
import no.integration.svv.exception.ValidationException;
import no.integration.svv.utils.validator.RequestValidator;
import no.vegvesen.felles.cct._1.Binarobjekt;
import no.vegvesen.felles.kodeliste.filtype._1.Filtype;
import no.vegvesen.saksarkiv.felles.typer._1.*;
import no.vegvesen.saksarkiv.journalpost.melding.leggtilnytt._1.LeggTilNyttDokumentJournalpostRequestType;
import org.springframework.http.MediaType;

import javax.activation.DataHandler;
import javax.mail.util.ByteArrayDataSource;
import java.io.UnsupportedEncodingException;
import java.math.BigInteger;

import static no.vegvesen.saksarkiv.felles.enkle._1.DokumentrolleType.HOVEDDOKUMENT;
import static no.vegvesen.saksarkiv.felles.enkle._1.DokumentrolleType.VEDLEGG;
import static no.vegvesen.saksarkiv.felles.enkle._1.DokumentstatusType.UNDER_ARBEID;
/**
 * @author vijayjadhav
 **/

/**
 * Geo-Endpoint to MimeWS Request mapping for NyDokument operation
 */
public class DokumentMap {


    /**
     * Request mapping from the endpoint to MimeWS server
     * */
    public static LeggTilNyttDokumentJournalpostRequestType endpointToMimeOppDok(final NyDokument input) throws UnsupportedEncodingException {


        RequestValidator.validateNyDokumentRequest(input);

        LeggTilNyttDokumentJournalpostRequestType leggTilNyttDokumentJournalpostRequestType = new LeggTilNyttDokumentJournalpostRequestType();
        RegistreringIdentifikatorRequestType registreringIdentifikatorRequestType = new RegistreringIdentifikatorRequestType();


        TilleggsdokumentType tilleggsdokumentType = new TilleggsdokumentType();
//        List<TilleggsdokumentType> tilleggsdokumentType = new ArrayList<TilleggsdokumentType>();
        DokumentMetadataType dokumentMetadataType = new DokumentMetadataType();
        if (input.getDokument() != null) {

            if (input.getDokument().getTilknyttetRegistreringSom() != null) {
                if (input.getDokument().getTilknyttetRegistreringSom().getKodeverdi().equalsIgnoreCase("H")) {
                    tilleggsdokumentType.setDokumentrolle(HOVEDDOKUMENT);

                }
                if (input.getDokument().getTilknyttetRegistreringSom().getKodeverdi().equalsIgnoreCase("V")) {
                    tilleggsdokumentType.setDokumentrolle(VEDLEGG);
                }
            }

            if (input.getDokument().getDokumenttype() != null) {
                dokumentMetadataType.setDokumenttype(input.getDokument().getDokumenttype().getKodeverdi());
            }
            dokumentMetadataType.setTittel(input.getDokument().getTittel());
            dokumentMetadataType.setDokumentstatus(UNDER_ARBEID);
            registreringIdentifikatorRequestType.setDokumentId(input.getDokument().getReferanseJournalpostSystemID());
            tilleggsdokumentType.setVedleggsindeks(new BigInteger("1"));
            dokumentMetadataType.setForfattere(new ForfatterType());
            dokumentMetadataType.setEier(new AnsvarsstedType());

        }
        tilleggsdokumentType.setMetadata(dokumentMetadataType);


        assert input.getDokument() != null;
        if (input.getDokument().getFil() != null) {
            Binarobjekt binarobjekt = new Binarobjekt();
            ByteArrayDataSource byteArrayDataSource = new ByteArrayDataSource(new byte[0], MediaType.APPLICATION_OCTET_STREAM_VALUE);
            byte[] bytes = input.getDokument().getFil().getBase64();
            if (bytes != null && bytes.length > 0) {
                byteArrayDataSource = new ByteArrayDataSource(bytes, MediaType.APPLICATION_OCTET_STREAM_VALUE);

            } else {
                throw new ValidationException("Fil", "base64", "File content is missing");
            }
            DataHandler dataHandler = new DataHandler(byteArrayDataSource);
            binarobjekt.setInnhold(dataHandler);
            binarobjekt.setFilnavn(input.getDokument().getFil().getFilnavn());
            binarobjekt.setMimetype(input.getDokument().getFil().getMimeType());

            if (input.getDokument().getFormat() != null) {
                binarobjekt.setFiltype(Filtype.valueOf(input.getDokument().getFormat().getKodeverdi().toUpperCase()));
                tilleggsdokumentType.setFil(binarobjekt);
            }
        }

        FagsystemType fagsystemType = getFagSystemMapping(input);
        leggTilNyttDokumentJournalpostRequestType.setFagsystem(fagsystemType);
        leggTilNyttDokumentJournalpostRequestType.getDokument().add(tilleggsdokumentType);


        leggTilNyttDokumentJournalpostRequestType.setRegistreringIdentifikator(registreringIdentifikatorRequestType);

        return leggTilNyttDokumentJournalpostRequestType;
    }

    private static FagsystemType getFagSystemMapping(NyDokument input) {
        FagsystemType fagsystemType = new FagsystemType();
        BrukeridentifikatorType brukeridentifikatorType = new BrukeridentifikatorType();
        if (input.getKontekst() != null) {
            fagsystemType.setFagsystemNavn(input.getKontekst().getKlientnavn());
            if(input.getKontekst().getKlientnavn().equalsIgnoreCase("Hr rekruttering") &&
                    (input.getKontekst().getSluttbruker()).contains("4100") ) {
                brukeridentifikatorType.setBrukerident(input.getKontekst().getSluttbruker().substring(4));
            }
            else{brukeridentifikatorType.setBrukerident(input.getKontekst().getSluttbruker());

            }
        }fagsystemType.setFagsystemSluttbruker(brukeridentifikatorType);
        return fagsystemType;
    }


    /**
     * Response mapping (MimeWS to endpoint)
     */

    public static NyDokumentResponse nyDokumentResponseToSuFa(final NyttVedleggKvitteringType mimeWS) {

        NyDokumentResponse nyDokumentResponse = new NyDokumentResponse();
        Dokument dokument = new Dokument();
        if (mimeWS != null) {

            if (mimeWS.getRegistrering() != null) {
                dokument.setReferanseJournalpostSystemID(mimeWS.getRegistrering().getDokumentId());
            }
            if (mimeWS.getDokument() != null && mimeWS.getDokument().size() > 0) {
                mimeWS.getDokument().forEach(item -> {
                    dokument.setSystemID(item.getDokumentId());
                });
            }
        }
        nyDokumentResponse.setReturn(dokument);
        return nyDokumentResponse;
    }
}